/* Generated by re2c 4.5.1 */

enum YYCONDTYPE {
	yycST_IN_SCRIPTING,
	yycST_LOOKING_FOR_PROPERTY,
	yycST_BACKQUOTE,
	yycST_DOUBLE_QUOTES,
	yycST_HEREDOC,
	yycST_LOOKING_FOR_VARNAME,
	yycST_VAR_OFFSET,
	yycSHEBANG,
	yycINITIAL,
	yycST_END_HEREDOC,
	yycST_NOWDOC
};
